%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SIMPLEX REGRESSION F. WAHL, T. ESPINASSE, Simplex Regression:
% Multivariable Parametric Regression under Shape Constraints Lyon1
% University, 2018
% http://math.univ-lyon1.fr/homes-www/wahl/download/index.php?file=simplexregression.zip
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 2 Dimensions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Declaration


clearvars
close all
% clc

warning off all

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

ModelTensor.nvar=4;
ModelTensor.typekernel={'poly','poly','poly','poly'};

nvar=ModelTensor.nvar;
exponents=[0,1,2];
ModelTensor.exponents=exponents;
for ivar=2:nvar
    ModelTensor.exponents=FW_ADDTENSOR180323(exponents,ModelTensor.exponents);
end
sel=sum(ModelTensor.exponents,2)>2;
ModelTensor.exponents(sel,:)=[];

% Display
verbose='yes';
% Figures
graph='yes';

% variables for which monotony is required :
% +1=increasing, -1=decreasing
% numvar=[1]; 
% numvar=[1 4]; 
% numvar=[1 4 2]; 
numvar=[1 4 2 3]; 
signevar=[-1 -1 +1 +1];
nnum=length(numvar);

% Noms
Nom_x={'x_1','x_2','x_3','x_4'};
Nom_y={'y'};
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% NO MOFICATION ALLOWED AFTER THIS LIMIT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Verification of the model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% number of variables 
nvar=ModelTensor.nvar;
if size(ModelTensor.exponents,1)==1 
    ModelTensor.exponents=ModelTensor.exponents';
end
if size(ModelTensor.exponents,2)~=nvar 
    fprintf('The field ''exponents'' and the number of variables are not consistent \n');
    return
elseif length(ModelTensor.typekernel) ~=nvar
    fprintf('number of kernels in the field ''typekernel'' # nb of variables %d \n',nvar);
    return
else
    fprintf('number of variables %d\n',nvar);
end

% number of coefficients
ncoeff=size(ModelTensor.exponents,1);
ModelTensor.ncoeff=ncoeff;
ModelTensor.mult=ones(1,ncoeff);

% exponents
expou=unique(ModelTensor.exponents,'rows','stable');
if size(expou,1) < ncoeff
   fprintf('duplicate row in the field ''exponents'' \n');
   return
end

% Models dimension 1
Model=cell(1,nvar);
for ivar=1:nvar
    Model{ivar}.exponents=unique(ModelTensor.exponents(:,ivar));
    Model{ivar}.ncoeff=length(Model{ivar}.exponents);
    Model{ivar}.mult=ones(1,Model{ivar}.ncoeff);
    Model{ivar}.typekernel=ModelTensor.typekernel{ivar};
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Data Input
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

filename='FW_HDS.txt';
Data=load(filename);

% Data
Xexp=Data(:,2:5);
yexp=Data(:,1);

% Xexp=Xexp(:,1:3);
% taille 
n=length(yexp);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Preparing and Treating
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Normalization between 0 and 1
% On normalise ente 0 et 1
xmin=min(Xexp);
xmax=max(Xexp);
X=(Xexp-repmat(xmin,n,1))./repmat(xmax-xmin,n,1);

yexpmin=0;
yexpmax=+1;
y=yexp;
ymin=min(y);
ymax=max(y);

[Equation0, Equationi, Contraintes]=FW_CC180529(X,y,numvar,signevar,ModelTensor,verbose);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Post-treatment
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
Aneq=Contraintes.matrice;

% counting the number of constraints
ncnt=size(Aneq,1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% equation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% normalization
X=(Xexp-repmat(xmin,n,1))./repmat(xmax-xmin,n,1);
y=(yexp-repmat(yexpmin,n,1))./repmat(yexpmax-yexpmin,n,1);

% Determination of the derivatives for each experimental point
ytest0=FW_CALCUL180529(X,Equation0);
ytesti=zeros(n,nnum);
for inum=1:nnum
    ytesti(:,inum)=FW_CALCUL180529(X,Equationi{inum});
end

% Determination of the derivatives at each corner
COINS=FW_COINS_HYPERCUBE_v2(zeros(1,nvar),ones(1,nvar));
ycoins0=FW_CALCUL180529(COINS,Equation0);
% derivatives
ycoinsi=zeros(2^nvar,nnum);
for inum=1:nnum
    ycoinsi(:,inum)=FW_CALCUL180529(COINS,Equationi{inum});
end


clear ke iv

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Verifying constraints
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('-----------------------\n');
fprintf('CONSTRAINTS VERIFIED AT THE CORNERS\n');
fprintf('-----------------------\n');
kdebut=0;

for inum=1:nnum
    fprintf('variable : %s \n',char(Nom_x(numvar(inum))));
    
    fprintf(' Number of Derivatives <0= %d\t',length(find(ycoinsi(:,inum) < 0)));
    fw=ycoinsi(ycoinsi(:,inum)<0,inum);
    fprintf(' (Min= %g) ',min(fw));
    fprintf(' Number of Derivatives >0= %d',length(find(ycoinsi(:,inum) > 0)));
    fw=ycoinsi(ycoinsi(:,inum)>0,inum);
    fprintf(' (Max= %g) \n',max(fw));
end

fprintf('-----------------------\n');
fprintf('CONSTRAINTS VERIFIED AT THE EXPERIMENTAL POINTS\n');
fprintf('-----------------------\n');


fprintf('Nbr variables = %d, size of the experimental data base= %d, nb of constraints=%d\n',nvar,n,ncnt);

for inum=1:nnum
    fprintf('variable : %s \n',char(Nom_x(numvar(inum))));
    kfin=ncnt;%Aneq_fin(inum);
    
    fprintf(' Number of first derivatives <0= %d\t',length(find(ytesti(:,inum) < 0)));
    dext=ytesti(ytesti(:,inum)<0,inum);
    fprintf(' (Min= %g) ',min(dext));
    fprintf(' Number of first derivatives >0= %d',length(find(ytesti(:,inum) > 0)));
    dext=ytesti(ytesti(:,inum)>0,inum);
    fprintf(' (Max= %g) \n',max(dext));
end
clear inum
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% RMSE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% denormalization
yhat=yexpmin+ytest0*(yexpmax-yexpmin);
% with constraints
res=yhat-yexp;
RMSE=sqrt(mean(res.^2));
fprintf('RMSE  %f\n\n',RMSE);

% without constraints
% Xmulti=FW_MakeX(X,Model);
Falpha=FW_MakeF180429(Model);
sel=sum(Falpha.exponents,2)>2;
Falpha.exponents(sel,:)=[];

Xmulti=x2fx(X,Falpha.exponents);
bmulti=(Xmulti'*Xmulti)\(Xmulti'*y);

yhatmulti=Xmulti*bmulti;
yhatmulti=yexpmin+yhatmulti*(yexpmax-yexpmin);
RMSEs=sqrt(mean((yhatmulti-yexp).^2));
fprintf('RMSE without constraints  %f\n\n',RMSEs);

% figure
if strcmp(graph,'yes')
    figure; hold on;
    
    yexpn=(yexp-ymin)/(ymax-ymin);
    resn=res/(ymax-ymin);
    plot(yexpn,resn,'*r');
    xlabel('y exp','FontSize',18); ylabel('y calculated- y exp','FontSize',18);
    title('*red=with constraints, +blue=without constraints','FontSize',18);
    
    plot(yexpn,(yhatmulti-yexp)/(ymax-ymin),'+b');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% figures 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if strcmp(graph,'yes')
    % The octopus
    % x randomly drawn between -1 and 1
    xtir=rand(1,nvar);
    xbase=[1 xtir];
    % without constraints
    for ivar1=1:nvar
        for ivar2=ivar1:nvar
            xbase=[xbase xbase(:,ivar1).*xbase(:,ivar2)];
        end
    end
    ysans=xbase*bmulti;
    ysansn=(ysans-ymin)/(ymax-ymin);
    % with constraints
    xbase=xtir;
    yavec=FW_CALCUL180529(xbase,Equation0);
    yavecn=(yavec-ymin)/(ymax-ymin);
    
    % regression
    ntir=100;
    xgrid=((0:(ntir-1))/(ntir-1))';
    
    Couleur='krbg';
    figure; hold on;
    for inum=1:nnum
        ivar=numvar(inum);
        
        Xgrid=ones(ntir,1)*[1 xtir];
        
        Xgrid(:,ivar+1)=xgrid;
        for ivar1=1:nvar
            for ivar2=ivar1:nvar
                Xgrid=[Xgrid Xgrid(:,ivar1).*Xgrid(:,ivar2)];
            end
        end
        % without constraints
        yhatmulti=Xgrid*bmulti;
        yhatmultin=(yhatmulti-ymin)/(ymax-ymin);
        plot(xgrid-xtir(ivar),yhatmultin,':','LineWidth',4,'Color',Couleur(inum));
        
        plot(xtir(ivar)-xtir(ivar),ysansn,'^','MarkerFaceColor',Couleur(inum),'MarkerSize',12);
        
        % with constraints
        yhat=zeros(ntir,1);
        for jco=1:ntir
            x=xtir;
            x(ivar)=xgrid(jco);
            
            yhat(jco)= FW_CALCUL180529(x,Equation0);
        end
        yhatn=(yhat-ymin)/(ymax-ymin);
        
        plot(xgrid-xtir(ivar),yhatn,'LineWidth',4,'Color',Couleur(inum));        
        plot(xtir(ivar)-xtir(ivar),yavecn,'o','MarkerFaceColor',Couleur(inum),'MarkerSize',12);
        
        
    end
    set(gca, 'XTick',[]);
    xlabel('black=x_1, red=x_2, blue=x_3, green=x_4','FontSize',18);%,'FontWeight','bold');
    ylabel('y calculated','FontSize',18);
    title('solid line=with constraints, dotted=without','FontSize',18);%,'FontWeight','bold');
    num2str(xtir)
end


return