%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 2 Dimensions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Declaration


clearvars
close all
% clc

warning off all


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Please uncomment one of these examples

% example 1
ModelTensor.nvar=1;
ModelTensor.typekernel={'poly'};
ModelTensor.exponents=0:5;
% variables for which monotony is required :
% +1=increasing, -1=decreasing
numvar=1;
signevar=+1;


% example 2
% ModelTensor.nvar=1;
% ModelTensor.typekernel={'exp'};
% ModelTensor.exponents=[0,0.5,1.2,2,2.1,2.5];
% % variables for which monotony is required :
% % +1=increasing, -1=decreasing
% numvar=1;
% signevar=+1;

% example 3
% ModelTensor.nvar=1;
% ModelTensor.typekernel={'exp'};
% ModelTensor.exponents=[0,-4,-3,-2,-1];
% % variables for which monotony is required :
% % +1=increasing, -1=decreasing
% numvar=1;
% signevar=+1;

% example 4
% ModelTensor.nvar=1;
% ModelTensor.typekernel={'power'};
% ModelTensor.exponents=[0,0.5,1.2,2,2.1,2.5];
% ModelTensor.offset=0.01;
% % variables for which monotony is required :
% % +1=increasing, -1=decreasing
% numvar=1;
% signevar=+1;

% example 5
% ModelTensor.nvar=2;
% ModelTensor.typekernel={'poly','poly'};
% ModelTensor.exponents=[
%     0,0 ; 0,1; 0,2; 0,3; 0,4
%     1,0 ; 1,1; 1,2; 1,3; 1,4
%     2,0 ; 2,1; 2,2; %2,3; 2,4
%     3,0 ; 3,1; 3,2; 3,3; 3,4
% ];
% % variables for which monotony is required :
% % +1=increasing, -1=decreasing
% numvar=[1,2];
% signevar=[+1,+1];

    
% Choose the data set. Note that with the power function, only the
% 'sigmoid' is possible, since Hawkin's example contains negative x values.
nomvrai='hawkins'; 
%nomvrai='sigmoid'; 

% Number of points to be selected : 
% if n==0 or if n > number of points in
% the data base (50 for both sigmoid and Hawkin's dataset), all the
% observed points are selected. Otherwise, n points are randomly selected
% from the data. Please select the number of points
n=8;

% Display
verbose='yes';
% Figures
graph='yes';

% Noms
Nom_x={'x_1','x_2'};
Nom_y={'y'};
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% NO MOFICATION ALLOWED AFTER THIS LIMIT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Verification of the model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% number of monotony constraints
nnum=length(numvar);
% number of variables 
nvar=ModelTensor.nvar;
if size(ModelTensor.exponents,1)==1 
    ModelTensor.exponents=ModelTensor.exponents';
end
if size(ModelTensor.exponents,2)~=nvar 
    fprintf('The field ''exponents'' and the number of variables are not consistent \n');
    return
elseif length(ModelTensor.typekernel) ~=nvar
    fprintf('number of kernels in the field ''typekernel'': %d \n',nvar);
    return
else
    fprintf('number of variables %d\n',nvar);
end

% number of coefficients
ncoeff=size(ModelTensor.exponents,1);
ModelTensor.ncoeff=ncoeff;
ModelTensor.mult=ones(1,ncoeff);

% exponents
expou=unique(ModelTensor.exponents,'rows','stable');
if size(expou,1) < ncoeff
   fprintf('duplicate row in the field ''exponents'' \n');
   return
end

% Models dimension 1
Model=FW_Makefj180429(ModelTensor);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Data Input
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
switch nvar
    case(1)
        switch nomvrai
            case('hawkins')
                nomfic='hawkins.txt';
                data=load(nomfic);
                
                if n>=size(data,1) || n <=0
                    tir=1:size(data,1);
                else
                    tir=randperm(size(data,1),n);
                end
                Xexp_ori=data(tir,1);
                yexp_ori=data(tir,2);
                n=length(yexp_ori);
                % normalization
                Xexpm=-1;
                XexpM=+1;
                Xexp=(Xexp_ori-Xexpm)/(XexpM-Xexpm);
                yexp=yexp_ori;
            case('sigmoid')
                nomfic='xyb50.txt';
                data=load(nomfic);
                
                if n>size(data,1) || n <=0
                    tir=1:size(data,1);
                else
                    tir=randperm(size(data,1),n);
                end
                
                Xexp_ori=data(tir,1);
                yexp_ori=data(tir,2);
                n=length(yexp_ori);
                % normalisation
                Xexpm=-1;
                XexpM=+1;
                Xexp=(Xexp_ori-Xexpm)/(XexpM-Xexpm);
                yexp=yexp_ori;
                if strcmp(Model{1}.typekernel,'power')
                    iexp=Xexp > Model{1}.offset;
                    Xexp=Xexp(iexp);
                    yexp=yexp(iexp);
                    n=length(yexp);
                end
        end
    case(2)
        nomvrai='2 variables';
        Xexp=rand(n,nvar);

        yvrai=1/2*(Xexp(:,2)+1/(1.4*pi)*sin(2*pi*Xexp(:,2))).*(1+(2*Xexp(:,1)-1).^3);
        yexp=yvrai+normrnd(0,0.1,size(yvrai));        
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Preparing and Treating
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ~isfield(ModelTensor,'offset')
    ModelTensor.offset=0;
end

% Normalization of X between 0 and 1
xmin=zeros(1,nvar)+ModelTensor.offset;
xmax=ones(1,nvar);
X=Xexp;

% number of coefficients
ncoeff=1;
for ivar=1:nvar
    expoi=Model{ivar}.exponents;
    ncoeff=ncoeff*length(expoi);
end
if ncoeff > n
    fprintf('number of points < number of coefficients \n');
    return
end

[Equation0, Equationi, Contraintes]=FW_CC180429dim2(X,yexp,numvar,signevar,ModelTensor,verbose);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Post-treatment
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
Aneq=Contraintes.matrice;

% counting the number of constraints
ncnt=size(Aneq,1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% equation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% normalization
X=(Xexp-repmat(xmin,n,1))./repmat(xmax-xmin,n,1);
% y=(yexp-repmat(yexpmin,n,1))./repmat(yexpmax-yexpmin,n,1);

% Determination of the derivatives for each experimental point
ytest0=FW_CALCUL180323(X,Equation0);
ytesti=zeros(n,nnum);
for inum=1:nnum
    ytesti(:,inum)=FW_CALCUL180323(X,Equationi{inum});
end

% corners
COINS=FW_COINS_HYPERCUBE_v2(zeros(1,nvar),ones(1,nvar));
for ivar=1:nvar
    if strcmp(ModelTensor.typekernel{ivar},'power')
        sel0=COINS(:,ivar)==0;
        COINS(sel0,ivar)=ModelTensor.offset;
    end
        
end

% responses at the corners
ycoins0=FW_CALCUL180323(COINS,Equation0);
% derivatives
ycoinsi=zeros(2^nvar,nnum);
for inum=1:nnum
    ycoinsi(:,inum)=FW_CALCUL180323(COINS,Equationi{inum});
end

clear ke iv

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Verifying the constraints
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('-----------------------\n');
fprintf('CONSTRAINTS VERIFIED AT THE CORNERS\n');
fprintf('-----------------------\n');
kdebut=0;

for inum=1:nnum
    fprintf('variable : %s \n',char(Nom_x(numvar(inum))));
    
    fprintf(' Number of Derivatives <0= %d\t',length(find(ycoinsi(:,inum) < 0)));
    fw=ycoinsi(ycoinsi(:,inum)<0,inum);
    fprintf(' (Min= %g) ',min(fw));
    fprintf(' Number of Derivatives >0= %d',length(find(ycoinsi(:,inum) > 0)));
    fw=ycoinsi(ycoinsi(:,inum)>0,inum);
    fprintf(' (Max= %g) \n',max(fw));
end

fprintf('-----------------------\n');
fprintf('CONSTRAINTS VERIFIED AT THE EXPERIMENTAL POINTS\n');
fprintf('-----------------------\n');


fprintf('Nbr variables = %d, size of the experimental data base= %d, nb of constraints=%d\n',nvar,n,ncnt);

for inum=1:nnum
    fprintf('variable : %s \n',char(Nom_x(numvar(inum))));
    kfin=ncnt;%Aneq_fin(inum);
    
    fprintf(' Number of first derivatives <0= %d\t',length(find(ytesti(:,inum) < 0)));
    dext=ytesti(ytesti(:,inum)<0,inum);
    fprintf(' (Min= %g) ',min(dext));
    fprintf(' Number of first derivatives >0= %d',length(find(ytesti(:,inum) > 0)));
    dext=ytesti(ytesti(:,inum)>0,inum);
    fprintf(' (Max= %g) \n',max(dext));
end
clear inum
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% RMSE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% with constraints
res=ytest0-yexp;
RMSE=sqrt(mean(res.^2));
fprintf('RMSE  %f\n\n',RMSE);

% without constraints
Xmulti=zeros(n,ncoeff);
for in=1:n
    x=1;
    for ivar=1:nvar
        x=kron(x,FW_MakeX(X(in,ivar),Model{ivar}));    
    end
    Xmulti(in,:)=x;
end
bmulti=(Xmulti'*Xmulti)\(Xmulti'*yexp);

yhat=Xmulti*bmulti;
% yhat=yexpmin+yhat*(yexpmax-yexpmin);
RMSEs=sqrt(mean((yhat-yexp).^2));
fprintf('RMSE without constraints  %f\n\n',RMSEs);

% figure
if strcmp(graph,'yes')
    figure; hold on;
    plot(yexp,res,'*r');
    xlabel('y exp','FontSize',14); ylabel('y calculated- y exp','FontSize',14);
    title('*red=with constraints, +blue=without constraints','FontSize',14);
    
    plot(yexp,yhat-yexp,'+b');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% figures 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if strcmp(graph,'yes')
    switch nvar
        case(1)
            FW_figure180323;
        case(2)
            for inum=1:nnum
                Vtgtes=Equationi{inum}.Liste_Vtgte;
                figure; plot(Vtgtes(:,1),Vtgtes(:,2),'.');
                hold on;
                index=unique(Vtgtes(:,nvar+1));
                for ix=1:length(index)
                    limits=Vtgtes(:,nvar+1)==index(ix);
                    x1max=max(Vtgtes(limits,1));
                    x1min=min(Vtgtes(limits,1));
                    x2max=max(Vtgtes(limits,2));
                    x2min=min(Vtgtes(limits,2));
                    rectangle('Position',[x1min,x2min,x1max-x1min,x2max-x2min]);
                end
                xlabel('x_1','FontSize',18);
                ylabel('x_2','FontSize',18);
                titre{1}=['constraint ', int2str(inum),':'];
                titre{2}='subdomains where a simplex is calculated';
                title(titre,'FontSize',18);
            end
    end
end

clear dummy ix
return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Bootstrap Hawkins
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if strcmp(nomvrai, 'hawkins')
    verbose='no';
    nb=1000;
    Xhw=2*X-1;
    yhat=4*Xhw.*(Xhw-2).^2.*(Xhw+0.5).^2.*(Xhw.^2+2);
    
    M_coeffb=zeros(ncoeff,nb);
    ybhat=zeros(n,nb);
    yb=zeros(n,nb);
    for ib=1:nb
        if ib==10*fix(ib/10)
            fprintf(' %d %', ib);
        end
        if ib==100*fix(ib/100)
            fprintf('\n');
        end
        
        % tirage avec remise, m out of n
        rtir=randn(n,1);
        yb(:,ib)=yhat-rtir;
        [Equationb]=FW_CC180429dim2(X,yb(:,ib),numvar,signevar,ModelTensor,verbose);
%        [Equationb]=FW_CC170110dim1PC(X,yb(:,ib),numvar,signevar,M_expo0,verbose);
        
        %     [Equationb]=FW_CC160910dim1(X,yb(:,ib),numvar,signevar,M_expo0);
        M_coeffb(:,ib)=Equationb.coeff;
        
        ytestb=FW_CALCUL180323(X,Equationb);
        ybhat(:,ib)=ytestb;
        %FW_CALCULv2(X,Equationb);
%         ybhat(:,ib)=yexpmin+ytestb*(yexpmax-yexpmin);
        
    end
    
    degrepoly=max(ModelTensor.exponents);
    for ic=0:degrepoly
        figure; plot(sort(M_coeffb(ic+1,:)),(1:nb)/nb)
        title(['distribution of coefficient ' int2str(ic)]);
    end
    
    return
    
    %% Sum of squares of Residuals
    SEb=1/n*sum((ybhat-yb).^2);
    fprintf('\n');
    degrepoly=max(ModelTensor.exponents);
    if degrepoly==4
        alpha0=1/16*M_coeff0(5)+1/8*M_coeff0(4)+1/4*M_coeff0(3)+1/2*M_coeff0(2)+M_coeff0(1);
        alpha1=4/16*M_coeff0(5)+3/8*M_coeff0(4)+2/4*M_coeff0(3)+1/2*M_coeff0(2);
        alpha2=6/16*M_coeff0(5)+3/8*M_coeff0(4)+1/4*M_coeff0(3);
        alpha3=4/16*M_coeff0(5)+1/8*M_coeff0(4);
        alpha4=1/16*M_coeff0(5);
        
        beta4=1/16*M_coeffb(5,:);
        beta3=4/16*M_coeffb(5,:)+1/8*M_coeffb(4,:);
        beta2=6/16*M_coeffb(5,:)+3/8*M_coeffb(4,:)+1/4*M_coeffb(3,:);
        beta1=4/16*M_coeffb(5,:)+3/8*M_coeffb(4,:)+2/4*M_coeffb(3,:)+1/2*M_coeffb(2,:);
        beta0=1/16*M_coeffb(5,:)+1/8*M_coeffb(4,:)+1/4*M_coeffb(3,:)+1/2*M_coeffb(2,:)+M_coeffb(1,:);
        
        beta4s=sort(beta4);beta3s=sort(beta3);beta2s=sort(beta2);beta1s=sort(beta1);beta0s=sort(beta0);
        [beta4s(50),alpha4,beta4s(950);
            beta3s(50),alpha3,beta3s(950);
            beta2s(50),alpha2,beta2s(950);
            beta1s(50),alpha1,beta1s(950);
            beta0s(50),alpha0,beta0s(950)];
    end
    if degrepoly==5
        alpha0=1/32*M_coeff0(6) +1/16*M_coeff0(5)+1/8*M_coeff0(4)+1/4*M_coeff0(3)+1/2*M_coeff0(2)+M_coeff0(1);
        alpha1=5/32*M_coeff0(6) +4/16*M_coeff0(5)+3/8*M_coeff0(4)+2/4*M_coeff0(3)+1/2*M_coeff0(2);
        alpha2=10/32*M_coeff0(6)+6/16*M_coeff0(5)+3/8*M_coeff0(4)+1/4*M_coeff0(3);
        alpha3=10/32*M_coeff0(6)+4/16*M_coeff0(5)+1/8*M_coeff0(4);
        alpha4=5/32*M_coeff0(6) +1/16*M_coeff0(5);
        alpha5=1/32*M_coeff0(6);
        
        beta0=1/32*M_coeffb(6,:) +1/16*M_coeffb(5,:)+1/8*M_coeffb(4,:)+1/4*M_coeffb(3,:)+1/2*M_coeffb(2,:)+M_coeffb(1,:);
        beta1=5/32*M_coeffb(6,:) +4/16*M_coeffb(5,:)+3/8*M_coeffb(4,:)+2/4*M_coeffb(3,:)+1/2*M_coeffb(2,:);
        beta2=10/32*M_coeffb(6,:)+6/16*M_coeffb(5,:)+3/8*M_coeffb(4,:)+1/4*M_coeffb(3,:);
        beta3=10/32*M_coeffb(6,:)+4/16*M_coeffb(5,:)+1/8*M_coeffb(4,:);
        beta4=5/32*M_coeffb(6,:) +1/16*M_coeffb(5,:);
        beta5=1/32*M_coeffb(6,:);
                
        beta5s=sort(beta5);beta4s=sort(beta4);beta3s=sort(beta3);beta2s=sort(beta2);
        beta1s=sort(beta1);beta0s=sort(beta0);
        [beta5s(50),alpha5,beta5s(950);
            beta4s(50),alpha4,beta4s(950);
            beta3s(50),alpha3,beta3s(950);
            beta2s(50),alpha2,beta2s(950);
            beta1s(50),alpha1,beta1s(950);
            beta0s(50),alpha0,beta0s(950)];
    end
end
return


