%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SIMPLEX REGRESSION F. WAHL, T. ESPINASSE, Simplex Regression:
% Multivariable Parametric Regression under Shape Constraints Lyon1
% University, 2018
% http://math.univ-lyon1.fr/homes-www/wahl/download/index.php?file=simplexregression.zip
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Declaration


clearvars
close all
% clc

warning off all


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Please uncomment one of these examples

% example 1
% ModelTensor.nvar=1;
% ModelTensor.typekernel={'poly'};
% ModelTensor.exponents=[0,1,2,3,4,5];
% % variables for which monotony is required :
% % +1=increasing, -1=decreasing
% numvar=1;
% signevar=+1;

% example 2
% ModelTensor.nvar=1;
% ModelTensor.typekernel={'exp'};
% ModelTensor.exponents=[0,0.5,1.2,2,2.1,2.5];
% % variables for which monotony is required :
% % +1=increasing, -1=decreasing
% numvar=1;
% signevar=+1;

% example 3
% ModelTensor.nvar=1;
% ModelTensor.typekernel={'exp'};
% ModelTensor.exponents=[0,-1.5,-1,-0.5,-0.25];
% % variables for which monotony is required :
% % +1=increasing, -1=decreasing
% numvar=1;
% signevar=+1;

% example 4
% ModelTensor.nvar=1;
% ModelTensor.typekernel={'power'};
% ModelTensor.exponents=[0,0.5,1.2,2,2.1,2.5];
% ModelTensor.offset=0.002;
% % variables for which monotony is required :
% % +1=increasing, -1=decreasing
% numvar=1;
% signevar=+1;

% example 5
ModelTensor.nvar=2;
ModelTensor.typekernel={'poly','poly'};
ModelTensor.exponents=[
    0,0 ; 0,1; 0,2; 0,3; 0,4
    1,0 ; 1,1; 1,2; 1,3; 1,4
    2,0 ; 2,1; 2,2; %2,3; 2,4
    3,0 ; 3,1; 3,2; 3,3; 3,4
];
% variables for which monotony is required :
% +1=increasing, -1=decreasing
numvar=[1,2];
signevar=[+1,+1];

    
% Choose the data set. Note that with the power function, only the
% 'sigmoid' is possible, since Hawkin's example contains negative x values.
% nomvrai='hawkins'; 
nomvrai='sigmoid'; 

% Number of points to be selected : 
% if n==0 or if n > number of points in
% the data base (50 for both sigmoid and Hawkin's dataset), all the
% observed points are selected. Otherwise, n points are randomly selected
% from the data. Please select the number of points
n=30;

% Display
verbose='yes';
% Figures
graph='yes';

% Noms
Nom_x={'x_1','x_2'};
Nom_y={'y'};
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% NO MOFICATION ALLOWED AFTER THIS LIMIT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Verification of the model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% number of monotony constraints
nnum=length(numvar);
% number of variables 
nvar=ModelTensor.nvar;
if size(ModelTensor.exponents,1)==1 
    ModelTensor.exponents=ModelTensor.exponents';
end
if size(ModelTensor.exponents,2)~=nvar 
    fprintf('The field ''exponents'' and the number of variables are not consistent \n');
    return
elseif length(ModelTensor.typekernel) ~=nvar
    fprintf('number of kernels in the field ''typekernel'': %d \n',nvar);
    return
else
    fprintf('number of variables %d\n',nvar);
end

% number of coefficients
ncoeff=size(ModelTensor.exponents,1);
ModelTensor.ncoeff=ncoeff;
ModelTensor.mult=ones(1,ncoeff);

% exponents
expou=unique(ModelTensor.exponents,'rows','stable');
if size(expou,1) < ncoeff
   fprintf('duplicate row in the field ''exponents'' \n');
   return
end

% Models dimension 1
Model=FW_Makefj180429(ModelTensor);

for ivar=1:nvar
    if strcmp(Model{ivar}.typekernel,'power')
        if ~isfield(ModelTensor, 'offset')
            fprintf('a power ''typekernel'' should have a positive offset  \n');
            return
        elseif ModelTensor.offset <=0
            fprintf('a power ''typekernel'' should have a positive offset  \n');
            return
        end
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Data Input
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
switch nvar
    case(1)
        switch nomvrai
            case('hawkins')
                nomfic='hawkins.txt';
                data=load(nomfic);
                
                if n>=size(data,1) || n <=0
                    tir=1:size(data,1);
                else
                    tir=randperm(size(data,1),n);
                end
                Xexp_ori=data(tir,1);
                yexp_ori=data(tir,2);
                n=length(yexp_ori);
                % normalization
                Xexpm=-1;
                XexpM=+1;
                Xexp=(Xexp_ori-Xexpm)/(XexpM-Xexpm);
                yexp=yexp_ori;
            case('sigmoid')
                nomfic='xyb50.txt';
                data=load(nomfic);
                
                if n>size(data,1) || n <=0
                    tir=1:size(data,1);
                else
                    tir=randperm(size(data,1),n);
                end
                
                Xexp_ori=data(tir,1);
                yexp_ori=data(tir,2);
                n=length(yexp_ori);
                % normalisation
                Xexpm=-1;
                XexpM=+1;
                Xexp=(Xexp_ori-Xexpm)/(XexpM-Xexpm);
                yexp=yexp_ori;
%                 if strcmp(Model{1}.typekernel,'power')
%                     iexp=Xexp > Model{1}.offset;
%                     Xexp=Xexp(iexp);
%                     yexp=yexp(iexp);
%                     n=length(yexp);
%                 end
        end
    case(2)
        nomvrai='2 variables';
        Xexp=rand(n,nvar);

        yvrai=1/2*(Xexp(:,2)+1/(1.4*pi)*sin(2*pi*Xexp(:,2))).*(1+(2*Xexp(:,1)-1).^3);
        yexp=yvrai+normrnd(0,0.1,size(yvrai));        
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Preparing and Treating
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Normalization of X between 0 and 1
% xmin=zeros(1,nvar)+ModelTensor.offset;
% xmax=ones(1,nvar);
X=Xexp;

% number of coefficients
ncoeff=1;
for ivar=1:nvar
    expoi=Model{ivar}.exponents;
    ncoeff=ncoeff*length(expoi);
end
if ncoeff > n
    fprintf('number of points < number of coefficients \n');
    return
end

[Equation0, Equationi, Contraintes]=FW_CC180529(X,yexp,numvar,signevar,ModelTensor,verbose);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Post-treatment
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
Aneq=Contraintes.matrice;

% counting the number of constraints
ncnt=size(Aneq,1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% equation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% normalization
% X=(Xexp-repmat(xmin,n,1))./repmat(xmax-xmin,n,1);
% y=(yexp-repmat(yexpmin,n,1))./repmat(yexpmax-yexpmin,n,1);

% Determination of the derivatives at each experimental point
ytest0=FW_CALCUL180529(X,Equation0);
ytesti=zeros(n,nnum);
for inum=1:nnum
    ytesti(:,inum)=FW_CALCUL180529(X,Equationi{inum});
end

% corners
COINS=FW_COINS_HYPERCUBE_v2(zeros(1,nvar),ones(1,nvar));
for ivar=1:nvar
    if strcmp(ModelTensor.typekernel{ivar},'power')
        sel0=COINS(:,ivar)==0;
        COINS(sel0,ivar)=ModelTensor.offset;
    end
        
end

% responses at the corners
ycoins0=FW_CALCUL180529(COINS,Equation0);
% derivatives
ycoinsi=zeros(2^nvar,nnum);
for inum=1:nnum
    ycoinsi(:,inum)=FW_CALCUL180529(COINS,Equationi{inum});
end

clear ke iv

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Verifying the constraints
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('-----------------------\n');
fprintf('CONSTRAINTS VERIFIED AT THE CORNERS\n');
fprintf('-----------------------\n');
kdebut=0;

for inum=1:nnum
    fprintf('variable : %s \n',char(Nom_x(numvar(inum))));
    
    fprintf(' Number of Derivatives <0= %d\t',length(find(ycoinsi(:,inum) < 0)));
    fw=ycoinsi(ycoinsi(:,inum)<0,inum);
    fprintf(' (Min= %g) ',min(fw));
    fprintf(' Number of Derivatives >0= %d',length(find(ycoinsi(:,inum) > 0)));
    fw=ycoinsi(ycoinsi(:,inum)>0,inum);
    fprintf(' (Max= %g) \n',max(fw));
end

fprintf('-----------------------\n');
fprintf('CONSTRAINTS VERIFIED AT THE EXPERIMENTAL POINTS\n');
fprintf('-----------------------\n');


fprintf('Nbr variables = %d, size of the experimental data base= %d, nb of constraints=%d\n',nvar,n,ncnt);

for inum=1:nnum
    fprintf('variable : %s \n',char(Nom_x(numvar(inum))));
    kfin=ncnt;%Aneq_fin(inum);
    
    fprintf(' Number of first derivatives <0= %d\t',length(find(ytesti(:,inum) < 0)));
    dext=ytesti(ytesti(:,inum)<0,inum);
    fprintf(' (Min= %g) ',min(dext));
    fprintf(' Number of first derivatives >0= %d',length(find(ytesti(:,inum) > 0)));
    dext=ytesti(ytesti(:,inum)>0,inum);
    fprintf(' (Max= %g) \n',max(dext));
end
clear inum
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% RMSE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% with constraints
res=ytest0-yexp;
RMSE=sqrt(mean(res.^2));
fprintf('RMSE  %f\n\n',RMSE);

% without constraints
Xmulti=zeros(n,ncoeff);
for in=1:n
    x=1;
    for ivar=1:nvar
        x=kron(x,FW_MakeX0529(X(in,ivar),Model{ivar}));    
    end
    Xmulti(in,:)=x;
end
bmulti=(Xmulti'*Xmulti)\(Xmulti'*yexp);

yhat=Xmulti*bmulti;
% yhat=yexpmin+yhat*(yexpmax-yexpmin);
RMSEs=sqrt(mean((yhat-yexp).^2));
fprintf('RMSE without constraints  %f\n\n',RMSEs);

% figure
if strcmp(graph,'yes')
    figure; hold on;
    plot(yexp,res,'*r');
    xlabel('y exp','FontSize',14); ylabel('y calculated- y exp','FontSize',14);
    title('*red=with constraints, +blue=without constraints','FontSize',14);
    
    plot(yexp,yhat-yexp,'+b');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% figures 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if strcmp(graph,'yes')
    switch nvar
        case(1)
            FW_figure180529;
        case(2)
            for inum=1:nnum
                Vtgtes=Equationi{inum}.Liste_Vtgte;
                figure; plot(Vtgtes(:,1),Vtgtes(:,2),'.');
                hold on;
                index=unique(Vtgtes(:,nvar+1));
                for ix=1:length(index)
                    limits=Vtgtes(:,nvar+1)==index(ix);
                    x1max=max(Vtgtes(limits,1));
                    x1min=min(Vtgtes(limits,1));
                    x2max=max(Vtgtes(limits,2));
                    x2min=min(Vtgtes(limits,2));
                    rectangle('Position',[x1min,x2min,x1max-x1min,x2max-x2min]);
                end
                xlabel('x_1','FontSize',18);
                ylabel('x_2','FontSize',18);
                titre{1}=['constraint ', int2str(inum),':'];
                titre{2}='subdomains where a simplex is calculated';
                title(titre,'FontSize',18);
            end
    end
end

clear dummy ix
return
