
function [Eq0, Eqinum, Cnt]=FW_CC180529(Xexp,yexp,numvar,signevar,ModelTensor,verbose)

% [Eq0, Eqi, Cnt]=FW_DETERMINATION_HypercubeE_140516(Xexp,yexp,numvar, signevar,M_expo0,typekernel,verbose);
% INPUT
% Xexp = Data X (dimension : (n,nx)
% yexp = responses corresponding to Xexp
% numvar = 1 when V=1
% signevar = sign of the monotony requirement
%   if =1, the response should be monotone increasing,
%   if =-1, decreasing otherwise
% M_Expo0 = monomial matrix in case of polynomial, or vector of exponents
% OUTPUT
% Eq0= structure containing the polynomial equation with 2 slots
%   coeff= vector of coefficients of the polynomial (dimension ncoeff) and
%   expo= matrix of exponents (ncoeff, nx+1)
% Eqi = structure containing the polynomial equation of the derivative,
% with the two slots
%   coeff is a matrix of coefficients. Its number of columns equals the
%   number of variables for which monotony is seeked
%   expo is a 3 dimensionnal matrix  (ncoeff, nx+1, nnum), where here
%   nnum=1
% Cnt = structure containing the  constraints. They are written as
% A*x <= b
%   Cnt.matrice = matrix of constraints = A while
%   Cnt.vecteur= vector of constraints = b.
%   Cnt.indices= not pertinent in this case
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SIMPLEX REGRESSION F. WAHL, T. ESPINASSE, Simplex Regression:
% Multivariable Parametric Regression under Shape Constraints Lyon1
% University, 2018
% http://math.univ-lyon1.fr/homes-www/wahl/download/index.php?file=simplexregression.zip
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Seuil
% seuil=1e-10;

% max number of optimizations
nbitermax=100;

% number of samples
n=size(Xexp,1);
% number of variables
nvar=ModelTensor.nvar;
% number of variables subject to monotony constraints
nnum=length(numvar);

if size(ModelTensor.exponents,1)==1 
    ModelTensor.exponents=ModelTensor.exponents';
end

% Models dimension 1
Model=FW_Makefj180429(ModelTensor);

% Construction of ModelTensorBis, resulting from the tensor product of the
% models of dimension 1 in Model.
% This structure may be different from ModelTensor
[ModelTensorBis]=FW_MakeF180429(Model);
ncoeffBis=ModelTensorBis.ncoeff;
ncoeff=ModelTensor.ncoeff;
if ncoeffBis > ncoeff
    for ic=1:ncoeffBis
        expoi=ModelTensorBis.exponents(ic,:);
        sumi=sum(abs(ModelTensor.exponents-ones(ncoeff,1)*expoi),2);
        if all(sumi)
            ModelTensorBis.mult(ic)=0;
        end
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Preparing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Normalization between 0 and 1
% X=(Xexp-repmat(min(Xexp),n,1))./repmat(max(Xexp)-min(Xexp),n,1);
X=Xexp;
yexpmin=min(yexp);
yexpmax=max(yexp);
y=(yexp-repmat(yexpmin,n,1))./repmat(yexpmax-yexpmin,n,1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% First Derivative
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ModelDeriv=cell(1,nnum);
ModelTensorBisD=cell(1,nnum);
for inum=1:nnum
    [ModelDi]=FW_M_derivee180529(Model{numvar(inum)});
    
    ModelDeriv{inum}=Model;
    ModelDeriv{inum}{numvar(inum)}=ModelDi;
    
    ModelTensorBisD{inum}=FW_MakeF180429(ModelDeriv{inum});
    ModelTensorBisD{inum}.mult=ModelTensorBisD{inum}.mult.*ModelTensorBis.mult;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Constraints
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Equality Constraintes
Aeq=[]; beq=[];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Monotony Constraints
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
nsindex=0;
for inum=1:nnum
    Simplexdim1=cell(1,nvar);
    for ivar=1:nvar
        Simplexdim1{ivar}=FW_Simplex180529(ModelDeriv{inum}{ivar},0,1);
        if ~isfield(Simplexdim1{ivar},'Vertices')
            fprintf('ERROR FW_CC180529 : the returned simplex can not be calculated\n');
            return
        end
        % List of values for which a simplex is tangent to the curve
        Simplexdim1{ivar}.Vtgtes=[0;1];
        
    end
    ModelTensorBisD{inum}.Simplexdim1=Simplexdim1;
        
    % Constraints on [0,1]
    nsindex=nsindex+1;
    [Sommets,Vtgtes,LVtgtes]=FW_Vertices180323(Simplexdim1,ModelDeriv{inum},nsindex);
    ModelTensorBisD{inum}.Sommets=Sommets;
    ns=size(Sommets,1);
    ModelTensorBisD{inum}.ns=ns;
    ModelTensorBisD{inum}.Vtgtes=Vtgtes;
    ModelTensorBisD{inum}.LVtgtes=LVtgtes;
    
    Aneqinum=Sommets.*(ones(size(Sommets,1),1)*ModelTensorBisD{inum}.mult');
    % Vtgtes = Vertices tangent to the manifold. The last column is the number
    % of the simplex.
    % LVtgtes = flag indicating whether a constraint corresponds to a vertex on
    % the manifold. If zero, the vertex is not on the manifold. Otherwise, the
    % flag is equal to ns, the number of the simplex to which it belongs.
        
    % increasing or decreasing
    if signevar == +1
        Aneqinum=-Aneqinum;
    end
    
    % saving the constraints
    ModelTensorBisD{inum}.Aneq=Aneqinum;
    nq=size(Aneqinum,1);
    ModelTensorBisD{inum}.nq=nq;
    % remembering the simplex
    LSimplex=zeros(nq,1);
    LSimplex(1:nq)=nsindex;
    ModelTensorBisD{inum}.LSimplex=LSimplex;
    
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Regression with constraints
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Setting up the whole matrix of constraints
nq=0;
ns=0;
for inum=1:nnum
    nq=nq+ModelTensorBisD{inum}.nq;
    ns=ns+ModelTensorBisD{inum}.ns;
end
Aneq=zeros(nq,ncoeffBis);
LVtgtes=zeros(nq,1);
LSimplex=zeros(nq,1);
ia=0;
Sommets=zeros(ns,ncoeffBis);
is=0;
for inum=1:nnum
    nqi=ModelTensorBisD{inum}.nq;
    Aneq(ia+(1:nqi),:)=ModelTensorBisD{inum}.Aneq;
    LVtgtes(ia+(1:nqi))=ModelTensorBisD{inum}.LVtgtes;
    LSimplex(ia+(1:nqi))=ModelTensorBisD{inum}.LSimplex;
    ia=ia+nqi;
    
    Sommets(is+(1:ModelTensorBisD{inum}.ns),:)=ModelTensorBisD{inum}.Sommets;
    is=is+ModelTensorBisD{inum}.ns;
end

bneq=zeros(nq,1);
alphaold=zeros(ncoeffBis,1);

Aneqnew=Aneq;
Aneqold=Aneq;
bneqold=bneq;
bneqnew=bneq;

% Sommets=Sommetsi;
Sommetsold=Sommets;

nqnew=size(Aneqnew,1);

RMSEold=inf;
nbiter=0;
isOK=1;
while isOK
    nbiter=nbiter+1;
    % optimization
    [alphanew,actives] = FW_qp180529(X,y,ModelTensorBis, Aeq,beq,Aneqnew,bneqnew);
    
    % RMSE
    Eq0.coeff=alphanew;
    Eq0.Model=Model;
    yhat=FW_CALCUL180529(X,Eq0);
    RMSEnew=sqrt(mean((yhat-y).^2));
%     RMSEoldbis=RMSEold;
    while RMSEnew>RMSEold && abs(RMSEnew-RMSEold) > 1e-06
        [alphanew,actives] = FW_qp180529(X,y,ModelTensorBis, Aeq,beq,Aneqnew,bneqnew,alphanew);
        Eq0.coeff=alphanew;
        Eq0.Model=Model;
        yhat=FW_CALCUL180529(X,Eq0);
        RMSEnew=sqrt(mean((yhat-y).^2));
        fprintf('iter=%d, RMSEold=%g-->RMSEnew=%g, RMSEold-RMSEnew=%g \n',...
            nbiter,RMSEold,RMSEnew,RMSEold-RMSEnew);
%         RMSEoldbis=RMSEnew;
    end
    
    % active constraints
    cr=actives;
    if strcmp(verbose,'yes')
        fprintf('iter=%d,nb of active cnt =%d, total nb of constraints =%d\n',nbiter,length(cr),nqnew );
    end
    
    if isempty(cr)
        % the end
        alpha=alphaold;
        Aneq=Aneqold;
        bneq=bneqold;
        Sommets=Sommetsold;
        break;
    end
    
    if strcmp(verbose,'yes')
        fprintf('iter=%d, RMSEold=%g-->RMSEnew=%g, RMSEold-RMSEnew=%g \n',...
            nbiter,RMSEold,RMSEnew,RMSEold-RMSEnew);
    end
    if RMSEnew > RMSEold ||  nbiter > nbitermax || abs(RMSEnew - RMSEold) < 1e-6
        % the end but there is one useless iteration
        alpha=alphaold;
        Aneq=Aneqold;
        bneq=bneqold;
        Sommets=Sommetsold;

        break;
    else
        RMSEold=RMSEnew;
    end
    
    % Remove the constraints corresponding to a point on the curve
    AOT=LVtgtes(cr)>0;
    cr(AOT)=[];
    ncr=length(cr);
    if ncr==0
        if strcmp(verbose,'yes')            
            fprintf('no more constraint active after removing the points on the manifold \n');
        end
    end
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%    
    % Loop
    %%%%%%%%%%%%%%%%%%%%%%%%%    
    CVIDE=true;

    debinum=0;
    for inum=1:nnum
        % find all the constraints attached to inum
        nqi=ModelTensorBisD{inum}.nq;
        cri=cr( cr> debinum & cr < debinum+nqi+1);
        ncri=length(cri);
        
        if ncri==0
            continue
        end
        % else
        % Remove the redundant constraints, i.e. the constraints corresponding
        % to a simplex which is already selected.
        Vnew=zeros(ncri,nvar);
        for ia=1:ncri
            % Find the simplex corresponding to the constraint.
            nsi=ModelTensorBisD{inum}.LSimplex(cr(ia));
            % Find the vertices in contact with the manifold
            selv=ModelTensorBisD{inum}.Vtgtes(:,nvar+1)==nsi;
            Vnew(ia,:)=mean(ModelTensorBisD{inum}.Vtgtes(selv,1:nvar));
        end
        [~,ia]=unique(Vnew,'rows');
        Vnew=Vnew(ia,:);
        cri=cri(ia);
        ncri=length(cri);
        
        [nv1,nv2]=size(ModelTensorBisD{inum}.Sommets);
        Snewi=zeros(ncri*2^nvar*nv1,nv2);
        Vtgtesnewi=zeros(ncri*2^nvar,nvar+1);
        LVtgtesnewi=zeros(ncri*2^nvar*nv1,1);
        LSimplexnewi=zeros(ncri*2^nvar*nv1,1);
        
        ici=0; la=0; icietla=0;
        ns_old=zeros(1,ncri);
       
        for ia=1:ncri
%           fprintf('length(Lsimplex)=%d,cri(ia)=%d\n',length(ModelTensorBisD{inum}.LSimplex),cri(ia));
            ns_old(ia)=ModelTensorBisD{inum}.LSimplex(cri(ia));
            % Find the vertices in contact with the manifold
            selv=ModelTensorBisD{inum}.Vtgtes(:,nvar+1)==ns_old(ia);
            Vmin=min(ModelTensorBisD{inum}.Vtgtes(selv,1:nvar));
            Vmax=max(ModelTensorBisD{inum}.Vtgtes(selv,1:nvar));

            % Create 2^nvar simplexes
            COINS=FW_COINS_HYPERCUBE_v2(Vmin,Vmax);
                        
            for icoins=1:2^nvar
                Simplexdim1=cell(1,nvar);
                isOKi=1;
                ivar=0;
                while isOKi && ivar < nvar
                    ivar=ivar+1;
%                    fprintf('iter=%d,cr=%d,inum=%d, ivar=%d,icoins=%d\n',nbiter,cri(ia),inum, ivar,icoins); 
                    Simplexdim1{ivar}=FW_Simplex180529(ModelDeriv{inum}{ivar},COINS(icoins,ivar),Vnew(ia,ivar));
                    if ~isfield(Simplexdim1{ivar},'Vertices')
                        fprintf('ERROR FW_CC180529 : the returned simplex can not be calculated\n');
                        return
                    end

                    % No vertices created
                    if ~isfield(Simplexdim1{ivar},'Vertices')
                        isOKi=0;
                    end
                end
                % next corner if necessary
                if ~isOKi
                    continue;
                end
                % else
                % create a new simplex
                nsindex=nsindex+1;
                [Snewicoins,Vtgtesnewicoins,LVtgtesnewicoins]=FW_Vertices180323(Simplexdim1,ModelDeriv{inum},nsindex);
                Snewi(ici+(1:size(Snewicoins,1)),:)=Snewicoins;
                ici=ici+size(Snewicoins,1);
                Vtgtesnewi(la+(1:size(Vtgtesnewicoins,1)),:)=Vtgtesnewicoins;
                la=la+size(Vtgtesnewicoins,1);
                LVtgtesnewi(icietla+(1:length(LVtgtesnewicoins)))=LVtgtesnewicoins;
                LSimplexnewi(icietla+(1:length(LVtgtesnewicoins)))=nsindex;
                icietla=icietla+length(LVtgtesnewicoins);
                
                CVIDE=false;
 
            end
        end %fin ia
        
        if ici > 0            
            % Remove the old simplexes
            % La il peut y avoir un pb : si l'un des simplexes n'a pas pu
            % etre cree.
            for ia=1:ncri
                selSommets=ModelTensorBisD{inum}.LSimplex==ns_old(ia);
                ModelTensorBisD{inum}.Sommets(selSommets,:)=[];
                ModelTensorBisD{inum}.LVtgtes(selSommets)=[];
                ModelTensorBisD{inum}.LSimplex(selSommets)=[];
                
                ModelTensorBisD{inum}.Vtgtes(ModelTensorBisD{inum}.Vtgtes(:,nvar+1)==ns_old(ia),:)=[];
            end
            
            % Add the new simplexes
            ModelTensorBisD{inum}.Sommets=[ModelTensorBisD{inum}.Sommets;Snewi(1:ici,:) ];
            ModelTensorBisD{inum}.Vtgtes=[ModelTensorBisD{inum}.Vtgtes;Vtgtesnewi(1:la,:) ];
            ModelTensorBisD{inum}.LVtgtes=[ModelTensorBisD{inum}.LVtgtes;LVtgtesnewi(1:icietla,:) ];
            ModelTensorBisD{inum}.LSimplex=[ModelTensorBisD{inum}.LSimplex;LSimplexnewi(1:icietla,:) ];       
        end            
    end
    
    if CVIDE
        % the end
        alpha=alphanew;
        Aneq=Aneqnew;
        bneq=bneqnew;
        break;
    end

    %  updating the constraints
    nqnew=0;
    for inum=1:nnum
        nqi=size(ModelTensorBisD{inum}.Sommets,1);
        ModelTensorBisD{inum}.nq=nqi;
        nqnew=nqnew+nqi;
    end
    Aneqnew=zeros(nqnew,ncoeffBis);
    Sommetsnew=zeros(nqnew,ncoeffBis);
    LVtgtes=zeros(nqnew,1);

    nq0=0;
    for inum=1:nnum
        nqi=ModelTensorBisD{inum}.nq;
        Sommetsnew(nq0+(1:nqi),:)=ModelTensorBisD{inum}.Sommets;
        LVtgtes(nq0+(1:nqi))=ModelTensorBisD{inum}.LVtgtes;
        
        % if signevar(inum)==+1, positivity constraints
        Aneqnew(nq0+(1:nqi),:)=-signevar(inum)*ModelTensorBisD{inum}.Sommets...
            .*(ones(nqi,1)*ModelTensorBisD{inum}.mult');

        nq0=nq0+nqi;
    end        
    % removal of possible redundant constraints
    [Aneqnew,ia]=unique(Aneqnew,'rows','stable');
    Sommetsnew=Sommetsnew(ia,:);
    LVtgtes=LVtgtes(ia,:);
        
    nqnew=size(Aneqnew,1);
    bneqnew=zeros(nqnew,1);

    % updating the vertices and saving
    Sommetsold=Sommets;
    Sommets=Sommetsnew;
    
    % updating the coefficients
    alphaold=alphanew;
    
    % saving the constraints
    Aneqold=Aneqnew;
       
end % fin isOK

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% final equation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% coefficients for the response
M_coeff0=alpha;
M_coeff0=M_coeff0*(yexpmax-yexpmin);

i0= (sum(ModelTensorBis.exponents,2)==0) & (ModelTensorBis.mult==1);
M_coeff0(i0)=M_coeff0(i0)+yexpmin;
Eq0.Model=Model;
Eq0.coeff=M_coeff0;

% coefficients for the derivative
Eqinum=cell(1,nnum);

for inum=1:nnum
    M_coeffinum=alpha.*ModelTensorBisD{inum}.mult;
    M_coeffinum=M_coeffinum*(yexpmax-yexpmin);
    M_coeffinum(1)=M_coeffinum(1);%+yexpmin;
    
    Eqinum{inum}.Model=ModelDeriv{inum};
    Eqinum{inum}.coeff=M_coeffinum;
    
    Eqinum{inum}.Liste_Vtgte=ModelTensorBisD{inum}.Vtgtes;
end

% Constraints
Cnt.matrice=Aneq;
Cnt.vecteur=bneq;
Cnt.indices=size(Aneq,1);            

Cnt.sommets=Sommets;



end



