
function [Z]=FW_MakeX0529(X,Model)
% [Z]=FW_MakeX(X,Model)
% calculates the 'f_{j,v}(x_v)' in the expression of 'F_{j}(X) (see the cited paper)
% Model corresponds to one of the  the model 'f_j', v in [1,V]
% X is a column vector of values of 'x_v'. 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SIMPLEX REGRESSION F. WAHL, T. ESPINASSE, Simplex Regression:
% Multivariable Parametric Regression under Shape Constraints Lyon1
% University, 2018
% http://math.univ-lyon1.fr/homes-www/wahl/download/index.php?file=simplexregression.zip
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if size(X,1)==1
    X=X';
end
n=size(X,1);

typekernel=Model.typekernel;

switch typekernel
    case 'poly'
        exponents=Model.exponents;
        ncoeff=length(exponents);
        Z=zeros(n,ncoeff);
        for ic=1:ncoeff
            Z(:,ic)=X.^exponents(ic);
        end
    case 'exp'
        exponents=Model.exponents;
        ncoeff=length(exponents);
        % only in the case of one variable (X is a column vector)
        Z=zeros(n,ncoeff);
        for ic=1:ncoeff
            Z(:,ic)=X*exponents(ic);
        end
        Z=exp(Z);
    case 'power'
        % only in the case of one variable (X is a column vector)
        exponents=Model.exponents;
        ncoeff=length(exponents);
        offset=Model.offset;
        X=X+offset;
        Z=zeros(size(X,1),ncoeff);
        for ic=1:ncoeff
            Z(:,ic)=X.^exponents(ic);
        end
end


end