
function [Simplex]=FW_Simploscexp1_180323(Model,a,b)

% [Sommets]=FW_Simploscexp1_180323(Model,a,b)
% Vertices of the osculating simplex between a and b for the exponential
% system. Each line of the matrix 'Sommets' is a vertex.
% Their determination is based on a linear system of equations
% The result is returned only in the case of a well-conditionned system (<
% condAseuil)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SIMPLEX REGRESSION F. WAHL, T. ESPINASSE, Simplex Regression:
% Multivariable Parametric Regression under Shape Constraints Lyon1
% University, 2018
% http://math.univ-lyon1.fr/homes-www/wahl/download/index.php?file=simplexregression.zip
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

condAseuil=1e15;

if nargin==1
    a=0;
    b=1;
end

% row vector
Ld0=Model.exponents;
mult=Model.mult;
if size(Ld0,1)>1
    Ld0=Ld0';
end

% The constant is skipped
Ld=Ld0(Ld0~=0);
indexSommets=1:length(Ld0);
indexSommets=indexSommets(Ld0 ~=0);

% number of exponents
nd=length(Ld);

% Matrices for the derivatives in a and b
Mda=zeros(nd);
Mdb=zeros(nd);
Mda(1,:)=exp(a*Ld);
Mdb(1,:)=exp(b*Ld);
coefd=ones(1,nd);
for id=2:nd
    coefd=coefd.*Ld;
    im=(coefd~=0);
    Mda(id,im)= coefd(im).* exp(a*Ld);
    Mdb(id,im)= coefd(im).* exp(b*Ld);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Matrix of the vertices
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% initialization
Sommets=zeros(nd,nd+1);

Sommets(:,1)=exp(a*Ld);
Sommets(:,(nd+1))=exp(b*Ld);

maxcondA=0;
for k=1:(nd-1)
    %  unitary orientation vectors of the k space in a (row vector)
    Ksa=Mda(2:(k+1),:);
    % unitary orientation vectors of the k space in a (row vector)
    Ksb=Mdb(2:(nd-k+1),:);
    % linear system
    A=[Ksa; Ksb];
    maxcondA=max(maxcondA,cond(A));
    ab=(Mdb(1,:)-Mda(1,:));
    lambda=linsolve(A',ab');
    
    Sommets(:,(k+1))=Mda(1,:)'+Ksa(1:k,:)' * lambda(1:k);    
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Results
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% maxcondA
if maxcondA< condAseuil
    %The constant is added if necessary
    indexcst=find(Ld0==0 & mult ~=0,1);
    if indexcst
        if indexcst==1
            Sommets=[ones(1,nd+1); Sommets];
            indexSommets=[indexcst indexSommets];
        elseif indexcst==length(Ld0)
            Sommets=[Sommets; ones(1,nd+1)];
            indexSommets=[indexSommets indexcst];
        elseif indexcst>1 && indexcst< length(Ld0)
            Sommets=[Sommets(1:indexcst-1,:); ones(1,nd+1);Sommets(indexcst+1:nd,:) ];
        end
    end
    Sommets=Sommets';
    
    % Return
    Simplex.Vertices=Sommets;
    Simplex.indexVertices=indexSommets;
else
%      maxcondA
    Simplex=[];


end
    


    

    



