
function [Simplex]=FW_Simploscpoly1_180405(Model,a,b)
% [Simplex]=FW_Simploscpoly1_180405(Model,a,b) 
% Gives the Vertices of the osculating simplex between a and b for the one
% dimensional Model.
% Their determination is based on a linear system of equations The result
% is returned only in the case of a well-conditionned system (< condAseuil) 
% Simplex is a structure with 2 fields.
% Simplex.vertices gives the vertices of the simplex. 
% Simplex.indexVertices lists the corresponding x
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SIMPLEX REGRESSION F. WAHL, T. ESPINASSE, Simplex Regression:
% Multivariable Parametric Regression under Shape Constraints Lyon1
% University, 2018
% http://math.univ-lyon1.fr/homes-www/wahl/download/index.php?file=simplexregression.zip
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


condAseuil=1e15;

if nargin==1
    a=0;
    b=1;
end

Ld0=Model.exponents;
mult=Model.mult;
% in row
if size(Ld0,1)>1
    Ld0=Ld0';
end

% The constants are skipped
Ld=Ld0(Ld0 ~=0);
indexSommets=1:length(Ld0);
indexSommets=indexSommets(Ld0 ~=0);

% number of degrees different from 0
nLd=length(Ld);

% Matrices for the derivatives in a and b
Mda=zeros(nLd);
Mdb=zeros(nLd);
Mda(1,:)=a.^Ld;
Mdb(1,:)=b.^Ld;
coefd=ones(1,nLd);
for id=2:nLd
    coefd=coefd.*(Ld-(id-2));
    im=(coefd~=0);
    Mda(id,im)= coefd(im).* (a.^(Ld(im)-(id-1)));
    Mdb(id,im)= coefd(im).* (b.^(Ld(im)-(id-1)));
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Matrix of the vertices
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% initialization
Sommets=zeros(nLd,nLd+1);

Sommets(:,1)=a.^Ld;
Sommets(:,(nLd+1))=b.^Ld;

maxcondA=0;
for k=1:(nLd-1)
    %  orientation vectors of the k space at a (row vector)
    Ksa=Mda(2:(k+1),:);
    % orientation vectors of the k space at b (row vector)
    Ksb=Mdb(2:(nLd-k+1),:);
    % linear system
    A=[Ksa; Ksb];
    ab=(Mdb(1,:)-Mda(1,:));
%     lambda=linsolve(A',ab');
    
    [U,S,V]=svd(A',0);
    S=diag(S);
    maxcondA=max(maxcondA,max(S)/min(S));

    Amoins1=V*diag(1./S)*U';    
    lambda=Amoins1*ab';
    
    Sommets(:,(k+1))=Mda(1,:)'+Ksa(1:k,:)' * lambda(1:k);    
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Results
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% maxcondA
if maxcondA< condAseuil
    %The constant is added if necessary
    indexcst=find(Ld0==0 & mult ~=0,1);
    if indexcst
        Sommets=[ones(1,nLd+1); Sommets];
        indexSommets=[indexcst indexSommets];
    end    
    Sommets=Sommets';
    
    % Output
    Simplex.Vertices=Sommets;
    Simplex.indexVertices=indexSommets;
else
%      maxcondA
    fprintf('ERROR in the determination of the simplex between %g and %g\n',a,b);
    Simplex=[];
end
    


    

    



