
function [Vertices,Vtgtes,LVtgtes]=FW_Vertices180323(Simplex,Model,ns)

% [Vertices,Vtgtes,LVtgtes]=FW_Vertices180323(Simplex,Model,ns)
% creates a simplex resulting from the product of tensors of simplex in one
% dimension
% Simplex is a cell array of nvar structures Simplex
% Model is a cell array of nvar structures Model
% ns is the number to affect to this simplex
% Vtgtes = Vertices tangent to the manifold. The last column is the number
% of the simplex.
% LVtgtes = flag indicating whether a constraint corresponds to a vertex on
% the manifold. If zero, the vertex is not on the manifold. Otherwise, the
% flag is equal to ns, the number of the simplex to which it belongs.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SIMPLEX REGRESSION F. WAHL, T. ESPINASSE, Simplex Regression:
% Multivariable Parametric Regression under Shape Constraints Lyon1
% University, 2018
% http://math.univ-lyon1.fr/homes-www/wahl/download/index.php?file=simplexregression.zip
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

nvar=length(Simplex);

% Vertices of the simplex resulting from the tensor product
s1=size(Simplex{1}.Vertices,1);
s2=length(Model{1}.exponents);
Vertices=zeros(s1,s2);
Vertices(:,Simplex{1}.indexVertices)=Simplex{1}.Vertices;
for ivar=2:nvar
    s1=size(Simplex{ivar}.Vertices,1);
    s2=length(Model{ivar}.exponents);

    Si=zeros(s1,s2);
    Si(:,Simplex{ivar}.indexVertices)=Simplex{ivar}.Vertices;
    Vertices=kron(Vertices,Si);
end

% Where is the manifold tangent to the simplex
Vtgtes=Simplex{1}.Vtgtes;
for ivar=2:nvar
    Vtgtes=FW_ADDTENSOR180323(Vtgtes,Simplex{ivar}.Vtgtes);
end
% the last column is the number of the simplex
Vtgtes=[Vtgtes,ns*ones(size(Vtgtes,1),1)];

% What are the vertices corresponding to Vtgtes ? For each variable, the
% first vertex and the last are the ends of the simplex. With the tensor
% product, it gives:
LVtgtes=zeros(size(Vertices,1),1);

LVtgtes(1)=ns;
si=length(Simplex{nvar}.indexVertices);
LVtgtes(si)=ns;

prodsi2=si;
for ivar=(nvar-1):-1:1
    si=length(Simplex{ivar}.indexVertices);
    
    prodsi1=prodsi2*si-prodsi2+1;
    LVtgtes(prodsi1:(prodsi2*si))=LVtgtes(1:prodsi2);

    prodsi2=prodsi2*si;
end

end



