
function [alpha, actives] = FW_qp180529(X,y,ModelTensor, Aeq,beq,Aneq,bneq,alphaini)
% [alpha, actives] = FW_qp180429(X,y,ModelTensor, Aeq,beq,Aneq,bneq,alphaini)
% returns the solution to the optimization problem 
% alpha is the solution
% actives is the list of active constraints.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SIMPLEX REGRESSION F. WAHL, T. ESPINASSE, Simplex Regression:
% Multivariable Parametric Regression under Shape Constraints Lyon1
% University, 2018
% http://math.univ-lyon1.fr/homes-www/wahl/download/index.php?file=simplexregression.zip
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% contraintes satisfaites
seuil=1e-10;

% number of points
n=size(X,1);
% number of variables
nvar=size(X,2);

% Models dimension 1
Model=FW_Makefj180429(ModelTensor);

% for the missing terms in ModelTensor
index0=ModelTensor.mult==0;
indexnon0=1:ModelTensor.ncoeff;
indexnon0(index0)=[];

% M=matrix of the model and Hessian matrix=M'*M
M=zeros(n,ModelTensor.ncoeff);
for in=1:n
    x=1;
    for ivar=1:nvar
        x=kron(x,FW_MakeX0529(X(in,ivar),Model{ivar}));    
    end
    M(in,:)=x;
end

% avoiding the zeros
M(:,index0)=[];
Aneq(:,index0)=[];
if ~isempty(Aeq)
    Aeq(:,index0)=[];
end

% singular value decomposition
[U, S, V]=svd(M,0);

% linear terms in quadprog
f=U'*y;

% least squares solution
betamc=V*diag(1./diag(S))*U'*y;

%%%%%%%%%%%%%%%%%%%%%%%%%%
% ATTENTION !!!!!
% check whether betamc is a solution 
%%%%%%%%%%%%%%%%%%%%%%%%%%
if all(Aneq*betamc<=bneq)
    actives=[];
    fprintf('solution betamc\n');
    alpha=zeros(ModelTensor.ncoeff,1);
    alpha(indexnon0)=betamc;
    return
end

% change of variable
if ~isempty(Aneq)
    Cneq=Aneq*V*diag(1./diag(S));
    dneq=bneq;
else
    Cneq=[];
    dneq=[];
end
if ~isempty(Aeq)
    Ceq=Aeq*V*diag(1./diag(S));
    deq=beq;
else
    Ceq=[];
    deq=[];
end

if nargin<8
    gamma=betamc;
else
    gamma=alphaini;
    gamma(index0)=[];
end

opts =optimoptions('quadprog','Algorithm','active-set','Display','off','MaxIterations',5000);
[beta] = quadprog(eye(length(betamc)),-f,Cneq,dneq,Ceq,deq,[],[],gamma,opts);

% back to the original variable
alpha0=V*diag(1./diag(S))*beta;

if ~isempty(Aneq)
%    cnr=sum(Aneq*alpha-bneq >seuil);
%     fprintf('nbcnt inegalite non respectees sur alpha (apres retour a l''echelle)=%d\n',cnr );
    % contraintes respectees
    cr=abs(Aneq*alpha0-bneq) <seuil;
    actives=1:size(Aneq,1);
    actives=actives(cr);
end
% if ~isempty(Aeq)
%    cnr=sum(Aeq*alpha-beq >seuil);
%     fprintf('nbcnt egalit? non respect?es sur alpha (apres retour a l''echelle)=%d\n',cnr );
% end

% Reincorporating the missing terms in the vector of coefficients
alpha=zeros(ModelTensor.ncoeff,1);
alpha(indexnon0)=alpha0;


end


